'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var G6 = _interopDefault(require('@antv/g6'));
var merge = _interopDefault(require('lodash/merge'));
var isArray = _interopDefault(require('lodash/isArray'));
var React = require('react');
var React__default = _interopDefault(React);
var pick = _interopDefault(require('lodash/pick'));
var cloneDeep = _interopDefault(require('lodash/cloneDeep'));
var isHotkey = _interopDefault(require('is-hotkey'));
var omit = _interopDefault(require('lodash/omit'));
var isPlainObject = _interopDefault(require('lodash/isPlainObject'));
var ReactDOM = _interopDefault(require('react-dom'));
var Popover = _interopDefault(require('antd/lib/popover'));
var delay = _interopDefault(require('lodash/delay'));
require('antd/lib/popover/style');
var classNames = _interopDefault(require('classnames'));

var EditorEvent;

(function (EditorEvent) {
  /** 调用命令之前触发 */
  EditorEvent["onBeforeExecuteCommand"] = "onBeforeExecuteCommand";
  /** 调用命令之后触发 */

  EditorEvent["onAfterExecuteCommand"] = "onAfterExecuteCommand";
  /** 改变画面状态触发 */

  EditorEvent["onGraphStateChange"] = "onGraphStateChange";
  /** 改变标签状态触发 */

  EditorEvent["onLabelStateChange"] = "onLabelStateChange";
})(EditorEvent || (EditorEvent = {}));

var GraphMode;

(function (GraphMode) {
  GraphMode["Default"] = "default";
  GraphMode["AddNode"] = "addNode";
  GraphMode["Readonly"] = "readonly";
})(GraphMode || (GraphMode = {}));

var GraphState;

(function (GraphState) {
  GraphState["NodeSelected"] = "nodeSelected";
  GraphState["EdgeSelected"] = "edgeSelected";
  GraphState["MultiSelected"] = "multiSelected";
  GraphState["CanvasSelected"] = "canvasSelected";
})(GraphState || (GraphState = {}));

var ItemType;

(function (ItemType) {
  ItemType["Node"] = "node";
  ItemType["Edge"] = "edge";
})(ItemType || (ItemType = {}));

var ItemState;

(function (ItemState) {
  ItemState["Active"] = "active";
  ItemState["ActiveAnchorPoints"] = "activeAnchorPoints";
  ItemState["Selected"] = "selected";
  ItemState["HighLight"] = "highLight";
  ItemState["Error"] = "error";
})(ItemState || (ItemState = {}));

var LabelState;

(function (LabelState) {
  LabelState["Hide"] = "hide";
  LabelState["Show"] = "show";
})(LabelState || (LabelState = {}));

var AnchorPointState;

(function (AnchorPointState) {
  AnchorPointState["Enabled"] = "enabled";
  AnchorPointState["Disabled"] = "disabled";
})(AnchorPointState || (AnchorPointState = {}));

var RendererType;

(function (RendererType) {
  RendererType["Canvas"] = "canvas";
  RendererType["Svg"] = "svg";
})(RendererType || (RendererType = {}));

var EditorCommand;

(function (EditorCommand) {
  /** 撤销 */
  EditorCommand["Undo"] = "undo";
  /** 重做 */

  EditorCommand["Redo"] = "redo";
  /** 添加 */

  EditorCommand["Add"] = "add";
  /** 更新 */

  EditorCommand["Update"] = "update";
  /** 删除 */

  EditorCommand["Remove"] = "remove";
  /** 复制 */

  EditorCommand["Copy"] = "copy";
  /** 粘贴 */

  EditorCommand["Paste"] = "paste";
  /** 粘贴到这里 */

  EditorCommand["PasteHere"] = "pasteHere";
  /** 放大 */

  EditorCommand["ZoomIn"] = "zoomIn";
  /** 缩小 */

  EditorCommand["ZoomOut"] = "zoomOut";
  /** 插入主题 */

  EditorCommand["Topic"] = "topic";
  /** 插入子主题 */

  EditorCommand["Subtopic"] = "subtopic";
  /** 收起 */

  EditorCommand["Fold"] = "fold";
  /** 展开 */

  EditorCommand["Unfold"] = "unfold";
})(EditorCommand || (EditorCommand = {}));

var GraphCommonEvent;

(function (GraphCommonEvent) {
  /** 单击鼠标左键或者按下回车键时触发 */
  GraphCommonEvent["onClick"] = "click";
  /** 双击鼠标左键时触发 */

  GraphCommonEvent["onDoubleClick"] = "dblclick";
  /** 鼠标移入元素范围内触发，该事件不冒泡，即鼠标移到其后代元素上时不会触发 */

  GraphCommonEvent["onMouseEnter"] = "mouseenter";
  /** 鼠标在元素内部移到时不断触发，不能通过键盘触发 */

  GraphCommonEvent["onMouseMove"] = "mousemove";
  /** 鼠标移出目标元素后触发 */

  GraphCommonEvent["onMouseOut"] = "mouseout";
  /** 鼠标移入目标元素上方，鼠标移到其后代元素上时会触发 */

  GraphCommonEvent["onMouseOver"] = "mouseover";
  /** 鼠标移出元素范围时触发，该事件不冒泡，即鼠标移到其后代元素时不会触发 */

  GraphCommonEvent["onMouseLeave"] = "mouseleave";
  /** 鼠标按钮被按下（左键或者右键）时触发，不能通过键盘触发 */

  GraphCommonEvent["onMouseDown"] = "mousedown";
  /** 鼠标按钮被释放弹起时触发，不能通过键盘触发 */

  GraphCommonEvent["onMouseUp"] = "mouseup";
  /** 用户右击鼠标时触发并打开上下文菜单 */

  GraphCommonEvent["onContextMenu"] = "contextmenu";
  /** 当拖拽元素开始被拖拽的时候触发的事件，此事件作用在被拖曳元素上 */

  GraphCommonEvent["onDragStart"] = "dragstart";
  /** 当拖拽元素在拖动过程中时触发的事件，此事件作用于被拖拽元素上 */

  GraphCommonEvent["onDrag"] = "drag";
  /** 当拖拽完成后触发的事件，此事件作用在被拖曳元素上 */

  GraphCommonEvent["onDragEnd"] = "dragend";
  /** 当拖曳元素进入目标元素的时候触发的事件，此事件作用在目标元素上 */

  GraphCommonEvent["onDragEnter"] = "dragenter";
  /** 当拖曳元素离开目标元素的时候触发的事件，此事件作用在目标元素上 */

  GraphCommonEvent["onDragLeave"] = "dragleave";
  /** 被拖拽的元素在目标元素上同时鼠标放开触发的事件，此事件作用在目标元素上 */

  GraphCommonEvent["onDrop"] = "drop";
  /** 按下键盘键触发该事件 */

  GraphCommonEvent["onKeyDown"] = "keydown";
  /** 释放键盘键触发该事件 */

  GraphCommonEvent["onKeyUp"] = "keyup";
  /** 当手指触摸屏幕时候触发，即使已经有一个手指放在屏幕上也会触发 */

  GraphCommonEvent["onTouchStart"] = "touchstart";
  /** 当手指在屏幕上滑动的时候连续地触发。在这个事件发生期间，调用 preventDefault() 事件可以阻止滚动。 */

  GraphCommonEvent["onTouchMove"] = "touchmove";
  /** 当手指从屏幕上离开的时候触发 */

  GraphCommonEvent["onTouchEnd"] = "touchend";
})(GraphCommonEvent || (GraphCommonEvent = {}));

var GraphNodeEvent;

(function (GraphNodeEvent) {
  /** 鼠标左键单击节点时触发 */
  GraphNodeEvent["onNodeClick"] = "node:click";
  /** 鼠标双击左键节点时触发 */

  GraphNodeEvent["onNodeDoubleClick"] = "node:dblclick";
  /** 鼠标移入节点时触发 */

  GraphNodeEvent["onNodeMouseEnter"] = "node:mouseenter";
  /** 鼠标在节点内部移到时不断触发，不能通过键盘触发 */

  GraphNodeEvent["onNodeMouseMove"] = "node:mousemove";
  /** 鼠标移出节点后触发 */

  GraphNodeEvent["onNodeMouseOut"] = "node:mouseout";
  /** 鼠标移入节点上方时触发 */

  GraphNodeEvent["onNodeMouseOver"] = "node:mouseover";
  /** 鼠标移出节点时触发 */

  GraphNodeEvent["onNodeMouseLeave"] = "node:mouseleave";
  /** 鼠标按钮在节点上按下（左键或者右键）时触发，不能通过键盘触发 */

  GraphNodeEvent["onNodeMouseDown"] = "node:mousedown";
  /** 节点上按下的鼠标按钮被释放弹起时触发，不能通过键盘触发 */

  GraphNodeEvent["onNodeMouseUp"] = "node:mouseup";
  /** 用户在节点上右击鼠标时触发并打开右键菜单 */

  GraphNodeEvent["onNodeContextMenu"] = "node:contextmenu";
  /** 当节点开始被拖拽的时候触发的事件，此事件作用在被拖曳节点上 */

  GraphNodeEvent["onNodeDragStart"] = "node:dragstart";
  /** 当节点在拖动过程中时触发的事件，此事件作用于被拖拽节点上 */

  GraphNodeEvent["onNodeDrag"] = "node:drag";
  /** 当拖拽完成后触发的事件，此事件作用在被拖曳节点上 */

  GraphNodeEvent["onNodeDragEnd"] = "node:dragend";
  /** 当拖曳节点进入目标元素的时候触发的事件，此事件作用在目标元素上 */

  GraphNodeEvent["onNodeDragEnter"] = "node:dragenter";
  /** 当拖曳节点离开目标元素的时候触发的事件，此事件作用在目标元素上 */

  GraphNodeEvent["onNodeDragLeave"] = "node:dragleave";
  /** 被拖拽的节点在目标元素上同时鼠标放开触发的事件，此事件作用在目标元素上 */

  GraphNodeEvent["onNodeDrop"] = "node:drop";
})(GraphNodeEvent || (GraphNodeEvent = {}));

var GraphEdgeEvent;

(function (GraphEdgeEvent) {
  /** 鼠标左键单击边时触发 */
  GraphEdgeEvent["onEdgeClick"] = "edge:click";
  /** 鼠标双击左键边时触发 */

  GraphEdgeEvent["onEdgeDoubleClick"] = "edge:dblclick";
  /** 鼠标移入边时触发 */

  GraphEdgeEvent["onEdgeMouseEnter"] = "edge:mouseenter";
  /** 鼠标在边上移到时不断触发，不能通过键盘触发 */

  GraphEdgeEvent["onEdgeMouseMove"] = "edge:mousemove";
  /** 鼠标移出边后触发 */

  GraphEdgeEvent["onEdgeMouseOut"] = "edge:mouseout";
  /** 鼠标移入边上方时触发 */

  GraphEdgeEvent["onEdgeMouseOver"] = "edge:mouseover";
  /** 鼠标移出边时触发 */

  GraphEdgeEvent["onEdgeMouseLeave"] = "edge:mouseleave";
  /** 鼠标按钮在边上按下（左键或者右键）时触发，不能通过键盘触发 */

  GraphEdgeEvent["onEdgeMouseDown"] = "edge:mousedown";
  /** 边上按下的鼠标按钮被释放弹起时触发，不能通过键盘触发 */

  GraphEdgeEvent["onEdgeMouseUp"] = "edge:mouseup";
  /** 用户在边上右击鼠标时触发并打开右键菜单 */

  GraphEdgeEvent["onEdgeContextMenu"] = "edge:contextmenu";
})(GraphEdgeEvent || (GraphEdgeEvent = {}));

var GraphCanvasEvent;

(function (GraphCanvasEvent) {
  /** 鼠标左键单击画布时触发 */
  GraphCanvasEvent["onCanvasClick"] = "canvas:click";
  /** 鼠标双击左键画布时触发 */

  GraphCanvasEvent["onCanvasDoubleClick"] = "canvas:dblclick";
  /** 鼠标移入画布时触发 */

  GraphCanvasEvent["onCanvasMouseEnter"] = "canvas:mouseenter";
  /** 鼠标在画布内部移到时不断触发，不能通过键盘触发 */

  GraphCanvasEvent["onCanvasMouseMove"] = "canvas:mousemove";
  /** 鼠标移出画布后触发 */

  GraphCanvasEvent["onCanvasMouseOut"] = "canvas:mouseout";
  /** 鼠标移入画布上方时触发 */

  GraphCanvasEvent["onCanvasMouseOver"] = "canvas:mouseover";
  /** 鼠标移出画布时触发 */

  GraphCanvasEvent["onCanvasMouseLeave"] = "canvas:mouseleave";
  /** 鼠标按钮在画布上按下（左键或者右键）时触发，不能通过键盘触发 */

  GraphCanvasEvent["onCanvasMouseDown"] = "canvas:mousedown";
  /** 画布上按下的鼠标按钮被释放弹起时触发，不能通过键盘触发 */

  GraphCanvasEvent["onCanvasMouseUp"] = "canvas:mouseup";
  /** 用户在画布上右击鼠标时触发并打开右键菜单 */

  GraphCanvasEvent["onCanvasContextMenu"] = "canvas:contextmenu";
  /** 当画布开始被拖拽的时候触发的事件，此事件作用在被拖曳画布上 */

  GraphCanvasEvent["onCanvasDragStart"] = "canvas:dragstart";
  /** 当画布在拖动过程中时触发的事件，此事件作用于被拖拽画布上 */

  GraphCanvasEvent["onCanvasDrag"] = "canvas:drag";
  /** 当拖拽完成后触发的事件，此事件作用在被拖曳画布上 */

  GraphCanvasEvent["onCanvasDragEnd"] = "canvas:dragend";
  /** 当拖曳画布进入目标元素的时候触发的事件，此事件作用在目标元素上 */

  GraphCanvasEvent["onCanvasDragEnter"] = "canvas:dragenter";
  /** 当拖曳画布离开目标元素的时候触发的事件，此事件作用在目标元素上 */

  GraphCanvasEvent["onCanvasDragLeave"] = "canvas:dragleave";
})(GraphCanvasEvent || (GraphCanvasEvent = {}));

var GraphCustomEvent;

(function (GraphCustomEvent) {
  /** 调用 add / addItem 方法之前触发 */
  GraphCustomEvent["onBeforeAddItem"] = "beforeadditem";
  /** 调用 add / addItem 方法之后触发 */

  GraphCustomEvent["onAfterAddItem"] = "afteradditem";
  /** 调用 remove / removeItem 方法之前触发 */

  GraphCustomEvent["onBeforeRemoveItem"] = "beforeremoveitem";
  /** 调用 remove / removeItem 方法之后触发 */

  GraphCustomEvent["onAfterRemoveItem"] = "afterremoveitem";
  /** 调用 update / updateItem 方法之前触发 */

  GraphCustomEvent["onBeforeUpdateItem"] = "beforeupdateitem";
  /** 调用 update / updateItem 方法之后触发 */

  GraphCustomEvent["onAfterUpdateItem"] = "afterupdateitem";
  /** 调用 showItem / hideItem 方法之前触发 */

  GraphCustomEvent["onBeforeItemVisibilityChange"] = "beforeitemvisibilitychange";
  /** 调用 showItem / hideItem 方法之后触发 */

  GraphCustomEvent["onAfterItemVisibilityChange"] = "afteritemvisibilitychange";
  /** 调用 setItemState 方法之前触发 */

  GraphCustomEvent["onBeforeItemStateChange"] = "beforeitemstatechange";
  /** 调用 setItemState 方法之后触发 */

  GraphCustomEvent["onAfterItemStateChange"] = "afteritemstatechange";
  /** 调用 refreshItem 方法之前触发 */

  GraphCustomEvent["onBeforeRefreshItem"] = "beforerefreshitem";
  /** 调用 refreshItem 方法之后触发 */

  GraphCustomEvent["onAfterRefreshItem"] = "afterrefreshitem";
  /** 调用 clearItemStates 方法之前触发 */

  GraphCustomEvent["onBeforeItemStatesClear"] = "beforeitemstatesclear";
  /** 调用 clearItemStates 方法之后触发 */

  GraphCustomEvent["onAfterItemStatesClear"] = "afteritemstatesclear";
  /** 布局前触发。调用 render 时会进行布局，因此 render 时会触发。或用户主动调用图的 layout 时触发 */

  GraphCustomEvent["onBeforeLayout"] = "beforelayout";
  /** 布局完成后触发。调用 render 时会进行布局，因此 render 时布局完成后会触发。或用户主动调用图的 layout 时布局完成后触发 */

  GraphCustomEvent["onAfterLayout"] = "afterlayout";
  /** 连线完成之前触发 */

  GraphCustomEvent["onBeforeConnect"] = "beforeconnect";
  /** 连线完成之后触发 */

  GraphCustomEvent["onAfterConnect"] = "afterconnect";
})(GraphCustomEvent || (GraphCustomEvent = {}));

var index = /*#__PURE__*/Object.freeze({
  __proto__: null,
  get EditorEvent () { return EditorEvent; },
  get GraphMode () { return GraphMode; },
  get GraphState () { return GraphState; },
  get ItemType () { return ItemType; },
  get ItemState () { return ItemState; },
  get LabelState () { return LabelState; },
  get AnchorPointState () { return AnchorPointState; },
  get RendererType () { return RendererType; },
  get EditorCommand () { return EditorCommand; },
  get GraphCommonEvent () { return GraphCommonEvent; },
  get GraphNodeEvent () { return GraphNodeEvent; },
  get GraphEdgeEvent () { return GraphEdgeEvent; },
  get GraphCanvasEvent () { return GraphCanvasEvent; },
  get GraphCustomEvent () { return GraphCustomEvent; }
});

const canvas = document.createElement('canvas');
const canvasContext = canvas.getContext('2d');
function optimizeMultilineText(text, font, maxRows, maxWidth) {
  canvasContext.font = font;

  if (canvasContext.measureText(text).width <= maxWidth) {
    return text;
  }

  let multilineText = [];
  let tempText = '';
  let tempTextWidth = 0;

  for (const char of text) {
    const {
      width
    } = canvasContext.measureText(char);

    if (tempTextWidth + width >= maxWidth) {
      multilineText.push(tempText);
      tempText = '';
      tempTextWidth = 0;
    }

    tempText += char;
    tempTextWidth += width;
  }

  if (tempText) {
    multilineText.push(tempText);
  }

  if (multilineText.length > maxRows) {
    const ellipsis = '...';
    const ellipsisWidth = canvasContext.measureText(ellipsis).width;
    let tempText = '';
    let tempTextWidth = 0;

    for (const char of multilineText[maxRows - 1]) {
      const {
        width
      } = canvasContext.measureText(char);

      if (tempTextWidth + width > maxWidth - ellipsisWidth) {
        break;
      }

      tempText += char;
      tempTextWidth += width;
    }

    multilineText = multilineText.slice(0, maxRows - 1).concat(`${tempText}${ellipsis}`);
  }

  return multilineText.join('\n');
}

const CONTENT_CLASS_NAME = 'node-content';
const LABEL_CLASS_NAME = 'node-label';
const baseNode = {
  options: {
    size: [120, 80],
    contentStyle: {
      fill: '#5487ea',
      fillOpacity: 0.08,
      stroke: '#5487ea',
      strokeOpacity: 0.65,
      radius: 6
    },
    labelStyle: {
      fill: '#000000',
      textAlign: 'center',
      textBaseline: 'middle'
    },
    stateStyles: {
      [ItemState.Active]: {
        wrapperStyle: {},
        contentStyle: {},
        labelStyle: {}
      },
      [ItemState.Selected]: {
        wrapperStyle: {},
        contentStyle: {},
        labelStyle: {}
      }
    }
  },

  getOptions(model) {
    return merge({}, this.options, this.getCustomConfig(model) || {}, model);
  },

  draw(model, group) {
    const keyShape = this.drawContent(model, group);
    this.drawLabel(model, group);
    return keyShape;
  },

  drawContent(model, group) {
    const [width, height] = this.getSize(model);
    const {
      contentStyle
    } = this.getOptions(model);
    const shape = group.addShape('rect', {
      className: CONTENT_CLASS_NAME,
      draggable: true,
      attrs: {
        x: 0,
        y: 0,
        width,
        height,
        ...contentStyle
      }
    });
    return shape;
  },

  drawLabel(model, group) {
    const [width, height] = this.getSize(model);
    const {
      labelStyle
    } = this.getOptions(model);
    const shape = group.addShape('text', {
      className: LABEL_CLASS_NAME,
      draggable: true,
      attrs: {
        x: width / 2,
        y: height / 2,
        text: model.label,
        ...labelStyle
      }
    });
    return shape;
  },

  setLabelText(model, group) {
    const shape = group.findByClassName(LABEL_CLASS_NAME);

    if (!shape) {
      return;
    }

    const [width] = this.getSize(model);
    const {
      fontStyle,
      fontWeight,
      fontSize,
      fontFamily
    } = shape.attr();
    const text = model.label;
    const font = `${fontStyle} ${fontWeight} ${fontSize}px ${fontFamily}`;
    shape.attr('text', optimizeMultilineText(text, font, 3, width));
  },

  update(model, item) {
    const group = item.getContainer();
    this.setLabelText(model, group);
  },

  setState(name, value, item) {
    const group = item.getContainer();
    const model = item.getModel();
    const states = item.getStates();
    [LABEL_CLASS_NAME].forEach(className => {
      const shape = group.findByClassName(className);
      const options = this.getOptions(model);
      const shapeName = className.split('-')[1];
      shape.attr({ ...options[`${shapeName}Style`]
      });
      states.forEach(state => {
        if (options.stateStyles[state] && options.stateStyles[state][`${shapeName}Style`]) {
          shape.attr({ ...options.stateStyles[state][`${shapeName}Style`]
          });
        }
      });
    });

    if (name === ItemState.Selected) {
      const contentShape = group.findByClassName(CONTENT_CLASS_NAME);

      if (value) {
        contentShape.attr({
          fillOpacity: 0.25,
          strokeOpacity: 0.92
        });
      } else {
        contentShape.attr({
          fillOpacity: 0.08,
          strokeOpacity: 0.65
        });
      }
    }

    if (this.afterSetState) {
      this.afterSetState(name, value, item);
    }
  },

  getSize(model) {
    const {
      size
    } = this.getOptions(model);

    if (!isArray(size)) {
      return [size, size];
    }

    return size;
  },

  getCustomConfig() {
    return {};
  },

  getAnchorPoints() {
    return [];
  }

};
G6.registerNode('base-node', baseNode);

const ANCHOR_POINT_NAME = 'anchorPoint';

const getAnchorPointDefaultStyle = (item, anchorPoint) => {
  const {
    width,
    height
  } = item.getKeyShape().getBBox();
  const [x, y] = anchorPoint;
  return {
    x: width * x,
    y: height * y,
    r: 3,
    lineWidth: 2,
    fill: '#FFFFFF',
    stroke: '#5AAAFF'
  };
};

const getAnchorPointDefaultDisabledStyle = (item, anchorPoint) => {
  const {
    width,
    height
  } = item.getKeyShape().getBBox();
  const [x, y] = anchorPoint;
  return {
    img: 'data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iOSIgaGVpZ2h0PSI4IiB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciPjxwYXRoIGQ9Ik0xLjUxNSAxLjE3Mmw1LjY1NyA1LjY1Nm0wLTUuNjU2TDEuNTE1IDYuODI4IiBzdHJva2U9IiNGRjYwNjAiIHN0cm9rZS13aWR0aD0iMS42IiBmaWxsPSJub25lIiBzdHJva2UtbGluZWNhcD0ic3F1YXJlIi8+PC9zdmc+',
    x: width * x - 4,
    y: height * y - 8,
    width: 8,
    height: 8
  };
};

function drawAnchorPoints(item, getAnchorPointStyle, getAnchorPointDisabledStyle) {
  const group = item.getContainer();
  const model = item.getModel();
  const anchorPoints = this.getAnchorPoints ? this.getAnchorPoints(model) : [];
  const anchorPointsState = item.get('anchorPointsState') || [];
  anchorPoints.forEach((anchorPoint, index) => {
    if (anchorPointsState[index] === AnchorPointState.Enabled) {
      group.addShape('circle', {
        name: ANCHOR_POINT_NAME,
        attrs: { ...getAnchorPointDefaultStyle(item, anchorPoint),
          ...getAnchorPointStyle(item, anchorPoint)
        },
        isAnchorPoint: true,
        anchorPointIndex: index,
        anchorPointState: AnchorPointState.Enabled
      });
    } else {
      group.addShape('image', {
        name: ANCHOR_POINT_NAME,
        attrs: { ...getAnchorPointDefaultDisabledStyle(item, anchorPoint),
          ...getAnchorPointDisabledStyle(item, anchorPoint)
        },
        isAnchorPoint: true,
        anchorPointIndex: index,
        anchorPointState: AnchorPointState.Disabled
      });
    }
  });
}

function removeAnchorPoints(item) {
  const group = item.getContainer();
  const anchorPoints = group.findAllByName(ANCHOR_POINT_NAME);
  anchorPoints.forEach(anchorPoint => {
    group.removeChild(anchorPoint);
  });
}

function setAnchorPointsState(name, value, item, getAnchorPointStyle = () => ({}), getAnchorPointDisabledStyle = () => ({})) {
  if (name !== ItemState.ActiveAnchorPoints) {
    return;
  }

  if (value) {
    drawAnchorPoints.call(this, item, getAnchorPointStyle, getAnchorPointDisabledStyle);
  } else {
    removeAnchorPoints.call(this, item);
  }
}

const taskNode = {
  afterSetState(name, value, item) {
    setAnchorPointsState.call(this, name, value, item);
  },

  getAnchorPoints() {
    return [[0.5, 0], [1, 0.5], [0.5, 1], [0, 0.5]];
  }

};
G6.registerNode('task-node', taskNode, 'base-node');

const EDGE_LABEL_CLASS_NAME = 'edge-label';
const EDGE_LABEL_WRAPPER_CLASS_NAME = 'edge-label-wrapper-label';
const flowEdge = {
  options: {
    style: {
      stroke: '#e5e5e5',
      lineWidth: 1,
      lineAppendWidth: 16,
      shadowColor: null,
      shadowBlur: 0,
      radius: 8,
      offset: 20,
      endArrow: {
        path: 'M 0,0 L 12,6 L 9,0 L 12,-6 Z',
        fill: '#e5e5e5'
      }
    },
    labelCfg: {
      style: {
        fill: '#000000',
        fontSize: 10
      }
    },
    stateStyles: {
      [ItemState.Selected]: {
        stroke: '#5aaaff',
        shadowColor: '#5aaaff',
        lineAppendWidth: 16,
        shadowBlur: 24,
        endArrow: {
          path: 'M 0,0 L 12,6 L 9,0 L 12,-6 Z',
          fill: '#5aaaff'
        }
      },
      [ItemState.HighLight]: {
        stroke: '#5aaaff',
        shadowColor: '#5aaaff',
        lineAppendWidth: 16,
        shadowBlur: 24,
        endArrow: {
          path: 'M 0,0 L 12,6 L 9,0 L 12,-6 Z',
          fill: '#5aaaff'
        }
      }
    }
  },

  getPath(points) {
    const startPoint = points[0];
    const endPoint = points[1];
    const arcRatio = 4;
    const arrowEndSpace = 9;
    let edgeX = startPoint.x;
    let edgeY = startPoint.y;
    let endX = endPoint.x;
    let endY = endPoint.y;
    const startControlPoint = [edgeX, edgeY];
    const endControlPoint = [endX, endY];
    const startAngel = this.getPointDirect(startPoint);
    const endAngel = this.getPointDirect(endPoint);
    const offsetLength = Math.sqrt(Math.pow(edgeX - endX, 2) + Math.pow(edgeY - endY, 2)) / arcRatio;
    startControlPoint[0] += (1 / startAngel < 0 ? -1 : 1) * Math.cos(startAngel) * offsetLength;
    startControlPoint[1] += -Math.sin(startAngel) * offsetLength; // svg坐标系倒置需要给y坐标加负号

    endControlPoint[0] += (1 / endAngel < 0 ? -1 : 1) * Math.cos(endAngel) * offsetLength;
    endControlPoint[1] += -Math.sin(endAngel) * offsetLength; // svg坐标系倒置需要给y坐标加负号

    if (endPoint.index === 0) {
      endY -= arrowEndSpace;
    } else if (endPoint.index === 1) {
      endX += arrowEndSpace;
    } else if (endPoint.index === 2) {
      endY += arrowEndSpace;
    } else if (endPoint.index === 3) {
      endX -= arrowEndSpace;
    }

    return [['M', startPoint.x, startPoint.y], ['C', startControlPoint[0], startControlPoint[1], endControlPoint[0], endControlPoint[1], endX, endY], ['L', endPoint.x, endPoint.y]];
  },

  getPointDirect(point) {
    let angel = 0;

    if (point.index === 0) {
      angel = Math.PI / 2;
    } else if (point.index === 1) {
      angel = -Math.PI;
    } else if (point.index === 2) {
      angel = -Math.PI / 2;
    } else if (point.index === 3) {
      angel = Math.PI;
    }

    return angel || 0;
  },

  createLabelWrapper(group, labelName) {
    const label = group.findByClassName(EDGE_LABEL_CLASS_NAME);
    const labelWrapper = group.findByClassName(EDGE_LABEL_WRAPPER_CLASS_NAME);

    if (!label) {
      return;
    }

    if (labelWrapper) {
      return;
    }

    if (!labelName) {
      return;
    }

    group.addShape('rect', {
      className: EDGE_LABEL_WRAPPER_CLASS_NAME,
      attrs: {
        fill: '#e1e5e8',
        radius: 2
      }
    });
    label.set('zIndex', 1);
    group.sort();
  },

  updateLabelWrapper(group, labelName) {
    const label = group.findByClassName(EDGE_LABEL_CLASS_NAME);
    const labelWrapper = group.findByClassName(EDGE_LABEL_WRAPPER_CLASS_NAME);

    if (!label || !labelName) {
      labelWrapper && labelWrapper.hide();
      return;
    } else {
      labelWrapper && labelWrapper.show();
    }

    if (!labelWrapper) {
      return;
    }

    const {
      minX,
      minY,
      width,
      height
    } = label.getBBox();
    labelWrapper.attr({
      x: minX - 5,
      y: minY - 3,
      width: width + 10,
      height: height + 6
    });
  },

  afterDraw(model, group) {
    this.createLabelWrapper(group, model.label);
    this.updateLabelWrapper(group, model.label);
  },

  afterUpdate(model, item) {
    const group = item.getContainer();
    this.createLabelWrapper(group, model.label);
    this.updateLabelWrapper(group, model.label);
  },

  setState(name, value, item) {
    const shape = item.get('keyShape');

    if (!shape) {
      return;
    }

    const {
      style,
      stateStyles
    } = this.options;
    const stateStyle = stateStyles[name];

    if (!stateStyle) {
      return;
    }

    if (value) {
      shape.attr({ ...style,
        ...stateStyle
      });
    } else {
      shape.attr(style);
    }
  }

}; // polyline

G6.registerEdge('flowEdge', flowEdge, 'line');

const EditorContext = React__default.createContext({
  graph: null,
  setGraph: () => {},
  commandManager: null,
  executeCommand: () => {}
});
const EditorContextProvider = EditorContext.Provider;
function useEditorContext() {
  return React__default.useContext(EditorContext);
}

/** 获取图表状态 */
const seenKeys = {};
const MULTIPLIER = Math.pow(2, 24);
/** 生成唯一标识 */

function guid() {
  let key;

  while (key === undefined || seenKeys.hasOwnProperty(key) || !isNaN(+key)) {
    key = Math.floor(Math.random() * MULTIPLIER).toString(32);
  }

  seenKeys[key] = true;
  return key;
}
function getGraphState(graph) {
  let graphState = GraphState.MultiSelected;
  const selectedNodes = getSelectedNodes(graph);
  const selectedEdges = getSelectedEdges(graph);

  if (selectedNodes.length === 1 && !selectedEdges.length) {
    graphState = GraphState.NodeSelected;
  }

  if (selectedEdges.length === 1 && !selectedNodes.length) {
    graphState = GraphState.EdgeSelected;
  }

  if (!selectedNodes.length && !selectedEdges.length) {
    graphState = GraphState.CanvasSelected;
  }

  return graphState;
}
/** 获取选中节点 */

function getSelectedNodes(graph) {
  return graph.findAllByState(ItemType.Node, ItemState.Selected);
}
/** 获取选中边线 */

function getSelectedEdges(graph) {
  return graph.findAllByState(ItemType.Edge, ItemState.Selected);
}
/** 清除选中状态 */

function clearSelectedState(graph, shouldUpdate = () => true) {
  const selectedNodes = getSelectedNodes(graph);
  const selectedEdges = getSelectedEdges(graph);
  executeBatch(graph, () => {
    [...selectedNodes, ...selectedEdges].forEach(item => {
      if (shouldUpdate(item)) {
        graph.setItemState(item, ItemState.Selected, false);
      }
    });
  });
}
/** 执行批量处理 */

function executeBatch(graph, execute) {
  const autoPaint = graph.get('autoPaint');
  graph.setAutoPaint(false);
  execute();
  graph.paint();
  graph.setAutoPaint(autoPaint);
}
/** 获取回溯路径 - Flow */

function getFlowRecallEdges(graph, node, targetIds = [], edges = []) {
  const inEdges = node.getInEdges();

  if (!inEdges.length) {
    return [];
  }

  inEdges.map(edge => {
    const sourceId = edge.getModel().source;
    const sourceNode = graph.findById(sourceId);
    edges.push(edge);
    const targetId = node.get('id');
    targetIds.push(targetId);

    if (!targetIds.includes(sourceId)) {
      getFlowRecallEdges(graph, sourceNode, targetIds, edges);
    }
  });
  return edges;
}
/** 设置选中元素 */

function setSelectedItems(graph, items) {
  executeBatch(graph, () => {
    const selectedNodes = getSelectedNodes(graph);
    const selectedEdges = getSelectedEdges(graph);
    [...selectedNodes, ...selectedEdges].forEach(node => {
      graph.setItemState(node, ItemState.Selected, false);
    });
    items.forEach(item => {
      graph.setItemState(item, ItemState.Selected, true);
    });
  });
  graph.emit(EditorEvent.onGraphStateChange, {
    graphState: getGraphState(graph)
  });
}

class CommandManager {
  constructor() {
    this.command = {};
    this.commandQueue = [];
    this.commandIndex = 0;

    this.register = (name, command) => {
      this.command[name] = { ...command,
        name
      };
    };

    this.execute = (graph, name, params) => {
      const Command = this.command[name];

      if (!Command) {
        return;
      }

      const command = Object.create(Command);
      command.params = { ...cloneDeep(Command.params),
        ...(params || {})
      };

      if (!command.canExecute(graph) || !command.shouldExecute(graph)) {
        return;
      }

      command.init(graph);
      graph.emit(EditorEvent.onBeforeExecuteCommand, {
        name: command.name,
        params: command.params
      });
      command.execute(graph);
      graph.emit(EditorEvent.onAfterExecuteCommand, {
        name: command.name,
        params: command.params
      });

      if (command.canUndo(graph)) {
        const {
          commandQueue,
          commandIndex
        } = this;
        commandQueue.splice(commandIndex, commandQueue.length - commandIndex, command);
        this.commandIndex += 1;
      }

      graph.emit(EditorEvent.onGraphStateChange, {
        graphState: getGraphState(graph)
      });
    };
  }

  canExecute(graph, name) {
    var _this$command$name;

    return (_this$command$name = this.command[name]) === null || _this$command$name === void 0 ? void 0 : _this$command$name.canExecute(graph);
  }

}

const Editor = props => {
  let commandManager = React.useMemo(() => new CommandManager(), []);
  const [graph, setStateGraph] = React.useState(null);
  const graphRef = React.useRef(null);
  const lastMouseDownTargetRef = React.useRef(null);
  const bindEvent = React.useCallback(graph => {
    const defaultEvent = () => {};

    graph.on(EditorEvent.onBeforeExecuteCommand, props[EditorEvent.onBeforeExecuteCommand] || defaultEvent);
    graph.on(EditorEvent.onAfterExecuteCommand, props[EditorEvent.onAfterExecuteCommand] || defaultEvent);
  }, []);
  React.useEffect(() => {
    const mouseDownHandler = e => {
      lastMouseDownTargetRef.current = e.target;
    };

    window.addEventListener(GraphCommonEvent.onMouseDown, mouseDownHandler);
    return () => {
      window.removeEventListener(GraphCommonEvent.onMouseDown, mouseDownHandler);
    };
  });
  const shouldTriggerShortcut = React.useCallback((graph, target) => {
    const renderer = graph.get('renderer');
    const canvasElement = graph.get('canvas').get('el');

    if (!target) {
      return false;
    }

    if (target === canvasElement) {
      return true;
    }

    if (renderer === RendererType.Svg) {
      if (target.nodeName === 'svg') {
        return true;
      }

      let parentNode = target.parentNode;

      while (parentNode && parentNode.nodeName !== 'BODY') {
        if (parentNode.nodeName === 'svg') {
          return true;
        } else {
          parentNode = parentNode.parentNode;
        }
      }

      return false;
    }
  }, []);
  const executeCommand = React.useCallback((name, params) => {
    const graph = graphRef.current;

    if (graph) {
      commandManager.execute(graph, name, params);
    }
  }, [commandManager]);
  const bindShortcut = React.useCallback(graph => {
    const handler = e => {
      if (!shouldTriggerShortcut(graph, lastMouseDownTargetRef.current)) {
        return;
      }

      Object.values(commandManager.command).some(command => {
        const {
          name,
          shortcuts
        } = command;
        const flag = shortcuts.some(shortcut => {
          if (isHotkey(shortcut, e)) {
            return true;
          }
        });

        if (flag) {
          if (commandManager.canExecute(graph, name)) {
            e.preventDefault();
            executeCommand(name);
            return true;
          }
        }

        return false;
      });
    };

    graph.on(GraphCommonEvent.onKeyDown, handler);
    return () => {
      graph.off(GraphCommonEvent.onKeyDown, handler);
    };
  }, [graph, commandManager, executeCommand]);
  const setGraph = React.useCallback(graph => {
    setStateGraph(graph);
    graphRef.current = graph;
    bindEvent(graph);
    bindShortcut(graph);
  }, [setStateGraph]);
  return /*#__PURE__*/React__default.createElement(EditorContextProvider, {
    value: {
      graph,
      setGraph,
      commandManager,
      executeCommand
    }
  }, /*#__PURE__*/React__default.createElement("div", Object.assign({}, pick(props, 'className', 'style')), props.children));
};

const redoCommand = {
  name: 'redo',
  params: {},

  canExecute(graph) {
    const commandManager = graph.get('commandManager');
    const {
      commandQueue,
      commandIndex
    } = commandManager;
    return commandIndex < commandQueue.length;
  },

  shouldExecute() {
    return true;
  },

  canUndo() {
    return false;
  },

  init() {},

  execute(graph) {
    const commandManager = graph.get('commandManager');
    const {
      commandQueue,
      commandIndex
    } = commandManager;
    commandQueue[commandIndex].execute(graph);
    commandManager.commandIndex += 1;
  },

  undo() {},

  shortcuts: ['mod+shift+z']
};

const undoCommand = {
  name: 'undo',
  params: {},

  canExecute(graph) {
    const commandManager = graph.get('commandManager');
    const {
      commandIndex
    } = commandManager;
    return commandIndex > 0;
  },

  shouldExecute() {
    return true;
  },

  canUndo() {
    return false;
  },

  init() {},

  execute(graph) {
    const commandManager = graph.get('commandManager');
    const {
      commandQueue,
      commandIndex
    } = commandManager;
    commandQueue[commandIndex - 1].undo(graph);
    commandManager.commandIndex -= 1;
  },

  undo() {},

  shortcuts: ['mod+z']
};

const baseCommand = {
  name: '',
  params: {},

  canExecute() {
    return true;
  },

  shouldExecute() {
    return true;
  },

  canUndo() {
    return true;
  },

  init() {},

  execute() {},

  undo() {},

  shortcuts: [],
  getSelectedNodes,
  getSelectedEdges,
  setSelectedItems,

  editSelectedNode(graph) {
    graph.emit(EditorEvent.onLabelStateChange, {
      labelState: LabelState.Show
    });
  }

};

const addCommand = { ...baseCommand,
  params: {
    type: ItemType.Node,
    model: {
      id: ''
    }
  },

  init() {
    const {
      model
    } = this.params;

    if (model.id) {
      return;
    }

    model.id = model.type + '-' + guid();
  },

  execute(graph) {
    const {
      type,
      model
    } = this.params;
    graph.add(type, model);
    this.setSelectedItems(graph, [model.id]);
  },

  undo(graph) {
    const {
      model
    } = this.params;
    graph.remove(model.id);
  }

};

const removeCommand = { ...baseCommand,
  params: {
    flow: {
      nodes: {},
      edges: {}
    },
    mind: {
      model: null,
      parent: ''
    }
  },

  canExecute(graph) {
    const selectedNodes = this.getSelectedNodes(graph);
    const selectedEdges = this.getSelectedEdges(graph);
    return !!(selectedNodes.length || selectedEdges.length);
  },

  init(graph) {
    const selectedNodes = this.getSelectedNodes(graph);
    const selectedEdges = this.getSelectedEdges(graph);
    const {
      nodes,
      edges
    } = this.params.flow;
    selectedNodes.forEach(node => {
      const nodeModel = node.getModel();
      const nodeEdges = node.getEdges();
      nodes[nodeModel.id] = nodeModel;
      nodeEdges.forEach(edge => {
        const edgeModel = edge.getModel();
        edges[edgeModel.id] = edgeModel;
      });
    });
    selectedEdges.forEach(edge => {
      const edgeModel = edge.getModel();
      edges[edgeModel.id] = edgeModel;
    });
  },

  execute(graph) {
    const {
      nodes,
      edges
    } = this.params.flow;
    executeBatch(graph, () => {
      [...Object.keys(nodes), ...Object.keys(edges)].forEach(id => {
        graph.removeItem(id);
      });
    });
  },

  undo(graph) {
    const {
      nodes,
      edges
    } = this.params.flow;
    executeBatch(graph, () => {
      Object.keys(nodes).forEach(id => {
        const model = nodes[id];
        graph.addItem(ItemType.Node, model);
      });
      Object.keys(edges).forEach(id => {
        const model = edges[id];
        graph.addItem(ItemType.Edge, model);
      });
    });
  },

  shortcuts: ['Delete', 'Backspace']
};

const updateCommand = { ...baseCommand,
  params: {
    id: '',
    originModel: {},
    updateModel: {},
    forceRefreshLayout: false
  },

  canExecute(graph) {
    const selectedNodes = this.getSelectedNodes(graph);
    const selectedEdges = this.getSelectedEdges(graph);
    return (selectedNodes.length || selectedEdges.length) && (selectedNodes.length === 1 || selectedEdges.length === 1);
  },

  init(graph) {
    const {
      id,
      updateModel
    } = this.params;
    const updatePaths = Object.keys(updateModel);
    this.params.originModel = pick(graph.findById(id).getModel(), updatePaths);
  },

  execute(graph) {
    const {
      id,
      updateModel,
      forceRefreshLayout
    } = this.params;
    graph.updateItem(id, updateModel);

    if (forceRefreshLayout) {
      graph.refreshLayout && graph.refreshLayout(false);
    }
  },

  undo(graph) {
    const {
      id,
      originModel
    } = this.params;
    graph.updateItem(id, originModel);
  }

};

class Global {
  constructor() {
    this.clipboard = {
      point: {
        x: 0,
        y: 0
      },
      models: []
    };
    this.component = {
      itemPanel: {
        model: null,
        delegateShapeClassName: `delegateShape_${guid()}`
      }
    };
    this.plugin = {
      itemPopover: {
        state: 'hide'
      },
      contextMenu: {
        state: 'hide'
      },
      editableLabel: {
        state: 'hide'
      }
    };
  }

}

var global = new Global();

const copyCommand = { ...baseCommand,

  canExecute(graph) {
    return !!this.getSelectedNodes(graph).length;
  },

  canUndo() {
    return false;
  },

  execute(graph) {
    const selectedNodes = this.getSelectedNodes(graph);
    global.clipboard.models = cloneDeep(selectedNodes.map(node => node.getModel()));
  },

  shortcuts: ['mod+c']
};

const pasteCommand = { ...baseCommand,
  params: {
    models: []
  },

  canExecute() {
    return !!global.clipboard.models.length;
  },

  init() {
    const {
      models
    } = global.clipboard;
    const offsetX = 10;
    const offsetY = 10;
    this.params = {
      models: models.map(model => {
        const {
          x,
          y
        } = model;
        return { ...model,
          id: model.type + '-' + guid(),
          x: x + offsetX,
          y: y + offsetY
        };
      })
    };
  },

  execute(graph) {
    const {
      models
    } = this.params;
    executeBatch(graph, () => {
      models.forEach(model => {
        graph.addItem(ItemType.Node, model);
      });
    });
    this.setSelectedItems(graph, models.map(model => model.id));
  },

  undo(graph) {
    const {
      models
    } = this.params;
    executeBatch(graph, () => {
      models.forEach(model => {
        graph.removeItem(model.id);
      });
    });
  },

  shortcuts: ['mod+v']
};

const pasteHereCommand = { ...pasteCommand,
  params: {
    models: []
  },

  init() {
    const {
      point,
      models
    } = global.clipboard;
    this.params = {
      models: models.map(model => {
        const {
          x,
          y
        } = model;
        const offsetX = point.x - x;
        const offsetY = point.y - y;
        return { ...model,
          id: model.type + '-' + guid(),
          x: x + offsetX,
          y: y + offsetY
        };
      })
    };
  },

  shortcuts: []
};

const DELTA = 0.05;
const zoomInCommand = { ...baseCommand,

  canUndo() {
    return false;
  },

  execute(graph) {
    const ratio = 1 + DELTA;
    const zoom = graph.getZoom() * ratio;
    const maxZoom = graph.get('maxZoom');

    if (zoom > maxZoom) {
      return;
    }

    graph.zoom(ratio);
  },

  shortcuts: ['mod+=']
};

const DELTA$1 = 0.05;
const zoomOutCommand = { ...baseCommand,

  canUndo() {
    return false;
  },

  execute(graph) {
    const ratio = 1 - DELTA$1;
    const zoom = graph.getZoom() * ratio;
    const minZoom = graph.get('minZoom');

    if (zoom < minZoom) {
      return;
    }

    graph.zoom(ratio);
  },

  shortcuts: ['mod+-']
};

var commands = {
  redo: redoCommand,
  undo: undoCommand,
  add: addCommand,
  remove: removeCommand,
  update: updateCommand,
  copy: copyCommand,
  paste: pasteCommand,
  pasteHere: pasteHereCommand,
  zoomIn: zoomInCommand,
  zoomOut: zoomOutCommand
};

class BehaviorManager {
  constructor() {
    this.behaviors = {};

    this.getRegisteredBehaviors = () => {
      const registeredBehaviors = {};
      Object.keys(this.behaviors).forEach(name => {
        const behavior = this.behaviors[name];
        const {
          graphMode = 'default'
        } = behavior;

        if (!registeredBehaviors[graphMode]) {
          registeredBehaviors[graphMode] = {};
        }

        registeredBehaviors[graphMode][name] = name;
      });
      return registeredBehaviors;
    };

    this.register = (name, behavior) => {
      this.behaviors[name] = behavior;
      G6.registerBehavior(name, behavior);
    };
  }

}

var behaviorManager = new BehaviorManager();

const clickItemBehavior = {
  getDefaultCfg() {
    return {
      multiple: true,
      keydown: false,
      keyCode: 17
    };
  },

  getEvents() {
    return {
      'node:click': 'handleItemClick',
      'edge:click': 'handleItemClick',
      'canvas:click': 'handleCanvasClick',
      'keydown': 'handleKeyDown',
      'keyup': 'handleKeyUp'
    };
  },

  handleItemClick({
    item
  }) {
    const {
      graph
    } = this;
    const isSelected = item.hasState(ItemState.Selected);

    if (this.multiple && this.keydown) {
      graph.setItemState(item, ItemState.Selected, !isSelected);
    } else {
      clearSelectedState(graph, selectedItem => {
        return selectedItem !== item;
      });

      if (!isSelected) {
        graph.setItemState(item, ItemState.Selected, true);
      }
    }

    graph.emit(EditorEvent.onGraphStateChange, {
      graphState: getGraphState(graph)
    });
  },

  handleCanvasClick() {
    const {
      graph
    } = this;
    clearSelectedState(graph);
    graph.emit(EditorEvent.onGraphStateChange, {
      graphState: GraphState.CanvasSelected
    });
  },

  handleKeyDown(e) {
    this.keydown = (e.keyCode || e.which) === this.keyCode;
  },

  handleKeyUp(e) {
    this.keydown = false;
  }

};
behaviorManager.register('click-item', clickItemBehavior);

const dragCanvasBehavior = {
  origin: null,
  keyCode: null,
  dragging: false,
  handleWindowMouseUp: null,

  getDefaultCfg() {
    return {
      allowKeyCode: [],
      notAllowKeyCode: [16]
    };
  },

  getEvents() {
    return {
      'canvas:dragstart': 'handleCanvasDragStart',
      'canvas:drag': 'handleCanvasDrag',
      'canvas:dragend': 'handleCanvasDragEnd',
      'canvas:mouseleave': 'handleCanvasMouseLeave',
      'canvas:contextmenu': 'handleCanvasContextMenu',
      keydown: 'handleKeyDown',
      keyup: 'handleKeyUp'
    };
  },

  canDrag() {
    const {
      keyCode,
      allowKeyCode,
      notAllowKeyCode
    } = this;
    let isAllow = !!!allowKeyCode.length;

    if (!keyCode) {
      return isAllow;
    }

    if (allowKeyCode.length && allowKeyCode.includes(keyCode)) {
      isAllow = true;
    }

    if (notAllowKeyCode.includes(keyCode)) {
      isAllow = false;
    }

    return isAllow;
  },

  updateViewport(e) {
    const {
      clientX,
      clientY
    } = e;
    const dx = clientX - this.origin.x;
    const dy = clientY - this.origin.y;
    this.origin = {
      x: clientX,
      y: clientY
    };
    this.graph.translate(dx, dy);
    this.graph.paint();
  },

  handleCanvasDragStart(e) {
    if (!this.shouldBegin.call(this, e)) {
      return;
    }

    if (!this.canDrag()) {
      return;
    }

    this.origin = {
      x: e.clientX,
      y: e.clientY
    };
    this.dragging = false;
  },

  handleCanvasDrag(e) {
    if (!this.shouldUpdate.call(this, e)) {
      return;
    }

    if (!this.canDrag()) {
      return;
    }

    if (!this.origin) {
      return;
    }

    if (!this.dragging) {
      this.dragging = true;
    } else {
      this.updateViewport(e);
    }
  },

  handleCanvasDragEnd(e) {
    if (!this.shouldEnd.call(this, e)) {
      return;
    }

    if (!this.canDrag()) {
      return;
    }

    this.origin = null;
    this.dragging = false;

    if (this.handleWindowMouseUp) {
      document.body.removeEventListener('mouseup', this.handleWindowMouseUp, false);
      this.handleWindowMouseUp = null;
    }
  },

  handleCanvasMouseLeave() {
    const canvasElement = this.graph.get('canvas').get('el');

    if (this.handleWindowMouseUp) {
      return;
    }

    this.handleWindowMouseUp = e => {
      if (e.target !== canvasElement) {
        this.handleCanvasDragEnd();
      }
    };

    document.body.addEventListener('mouseup', this.handleWindowMouseUp, false);
  },

  handleCanvasContextMenu() {
    this.origin = null;
    this.dragging = false;
  },

  handleKeyDown(e) {
    this.keyCode = e.keyCode || e.which;
  },

  handleKeyUp() {
    this.keyCode = null;
  }

};
behaviorManager.register('drag-canvas', dragCanvasBehavior);

const hoverItemBehavior = {
  getEvents() {
    return {
      'node:mouseenter': 'handleItemMouseenter',
      'edge:mouseenter': 'handleItemMouseenter',
      'node:mouseleave': 'handleItemMouseleave',
      'edge:mouseleave': 'handleItemMouseleave'
    };
  },

  handleItemMouseenter({
    item
  }) {
    const {
      graph
    } = this;
    graph.setItemState(item, ItemState.Active, true);
  },

  handleItemMouseleave({
    item
  }) {
    const {
      graph
    } = this;
    graph.setItemState(item, ItemState.Active, false);
  }

};
behaviorManager.register('hover-item', hoverItemBehavior);

const recallEdgeBehavior = {
  edgeIds: [],

  getEvents() {
    return {
      'node:click': 'handleNodeClick',
      'edge:click': 'handleEdgeClick',
      'canvas:click': 'handleCanvasClick'
    };
  },

  setHighLightState(edges) {
    const {
      graph
    } = this;
    this.clearHighLightState();
    executeBatch(graph, () => {
      edges.forEach(item => {
        graph.setItemState(item, ItemState.HighLight, true);
      });
    });
    this.edgeIds = edges.map(edge => edge.get('id'));
  },

  clearHighLightState() {
    const {
      graph
    } = this;
    executeBatch(graph, () => {
      this.edgeIds.forEach(id => {
        const item = graph.findById(id);

        if (item && !item.destroyed) {
          graph.setItemState(item, ItemState.HighLight, false);
        }
      });
    });
    this.edgeIds = [];
  },

  handleNodeClick({
    item
  }) {
    const {
      graph
    } = this;
    let edges = [];
    edges = getFlowRecallEdges(graph, item);
    this.setHighLightState(edges);
  },

  handleEdgeClick() {
    this.clearHighLightState();
  },

  handleCanvasClick() {
    this.clearHighLightState();
  }

};
behaviorManager.register('recall-edge', recallEdgeBehavior);

const GraphComponent = props => {
  const containerRef = React.useRef();
  const graphRef = React.useRef(null);
  let {
    setGraph,
    commandManager
  } = useEditorContext();
  React.useEffect(() => {
    init();
    bindEvent();
  }, []);

  const init = () => {
    if (containerRef.current) {
      const container = containerRef.current;
      const {
        clientWidth = 0,
        clientHeight = 0
      } = container;
      const actConfig = merge(props.config, {
        container: containerRef.current,
        width: clientWidth,
        height: clientHeight
      });
      let graph = new G6.Graph(actConfig);
      graphRef.current = graph;
      const data = { ...props.data
      };

      if (data) {
        props.parseData(data);
        graph.data(data);
      }

      graph.render();
      setGraph(graph); // 设置命令管理器

      graph.set('commandManager', commandManager);
      Object.keys(commands).forEach(name => {
        commandManager.register(name, commands[name]);
      });
    }
  };

  const bindEvent = () => {
    if (graphRef.current) {
      const graph = graphRef.current;
      const events = { ...GraphCommonEvent,
        ...GraphNodeEvent,
        ...GraphEdgeEvent,
        ...GraphCanvasEvent,
        ...GraphCustomEvent
      };
      Object.keys(events).forEach(event => {
        if (typeof props[event] === 'function') {
          graph.on(events[event], props[event]);
        }
      });
    }
  };

  return /*#__PURE__*/React__default.createElement("div", Object.assign({
    ref: containerRef
  }, pick(props, ['className', 'style'])));
};

const dragAddNodeBehavior = {
  shape: null,
  graphMode: GraphMode.AddNode,

  getEvents() {
    return {
      'canvas:mouseenter': 'handleCanvasMouseEnter',
      mousemove: 'handleMouseMove',
      mouseup: 'handleMouseUp'
    };
  },

  handleCanvasMouseEnter(e) {
    const {
      graph,
      shape
    } = this;

    if (shape) {
      return;
    }

    const group = graph.get('group');
    const model = global.component.itemPanel.model;
    const {
      size = 100
    } = model;
    let width = 0;
    let height = 0;

    if (isArray(size)) {
      width = size[0];
      height = size[1];
    } else {
      width = size;
      height = size;
    }

    const x = e.x - width / 2;
    const y = e.y - height / 2;
    this.shape = group.addShape('rect', {
      className: global.component.itemPanel.delegateShapeClassName,
      attrs: {
        x,
        y,
        width,
        height,
        fill: '#f3f9ff',
        fillOpacity: 0.5,
        stroke: '#1890ff',
        strokeOpacity: 0.9,
        lineDash: [5, 5]
      }
    });
    graph.paint();
  },

  handleMouseMove(e) {
    const {
      graph
    } = this;
    const {
      width,
      height
    } = this.shape.getBBox();
    const x = e.x - width / 2;
    const y = e.y - height / 2;
    this.shape.attr({
      x,
      y
    });
    graph.paint();
  },

  handleMouseUp(e) {
    const {
      graph
    } = this;
    const {
      width,
      height
    } = this.shape.getBBox();
    let x = e.x;
    let y = e.y;
    const model = global.component.itemPanel.model;

    if (model.center === 'topLeft') {
      x -= width / 2;
      y -= height / 2;
    }

    this.shape.remove(true);
    const commandManager = graph.get('commandManager');
    commandManager.execute(graph, EditorCommand.Add, {
      type: ItemType.Node,
      model: {
        id: model.type + '-' + guid(),
        x,
        y,
        ...model
      }
    });
  }

};
behaviorManager.register('drag-add-node', dragAddNodeBehavior);

const dragAddEdgeBehavior = {
  edge: null,

  getDefaultCfg() {
    return {
      edgeType: 'flowEdge',
      getAnchorPointStateOfSourceNode: () => AnchorPointState.Enabled,
      getAnchorPointStateOfTargetNode: () => AnchorPointState.Enabled
    };
  },

  getEvents() {
    return {
      'node:mouseenter': 'handleNodeMouseEnter',
      'node:mouseleave': 'handleNodeMouseLeave',
      'node:mousedown': 'handleNodeMouseDown',
      mousemove: 'handleMouseMove',
      mouseup: 'handleMouseUp'
    };
  },

  isEnabledAnchorPoint(e) {
    const {
      target
    } = e;
    return !!target.get('isAnchorPoint') && target.get('anchorPointState') === AnchorPointState.Enabled;
  },

  isNotSelf(e) {
    const {
      edge
    } = this;
    const {
      item
    } = e;
    return item.getModel().id !== edge.getSource().getModel().id;
  },

  getTargetNodes(sourceId) {
    const {
      graph
    } = this;
    const nodes = graph.getNodes();
    return nodes.filter(node => node.getModel().id !== sourceId);
  },

  canFindTargetAnchorPoint(e) {
    return this.isEnabledAnchorPoint(e) && this.isNotSelf(e);
  },

  shouldAddDelegateEdge(e) {
    return this.isEnabledAnchorPoint(e);
  },

  shouldAddRealEdge() {
    const {
      edge
    } = this;
    const target = edge.getTarget();
    return !isPlainObject(target);
  },

  handleNodeMouseEnter(e) {
    const {
      graph,
      getAnchorPointStateOfSourceNode
    } = this;
    const sourceNode = e.item;
    const sourceAnchorPoints = sourceNode.getAnchorPoints();
    const sourceAnchorPointsState = [];
    sourceAnchorPoints.forEach(sourceAnchorPoint => {
      sourceAnchorPointsState.push(getAnchorPointStateOfSourceNode(sourceNode, sourceAnchorPoint));
    });
    sourceNode.set('anchorPointsState', sourceAnchorPointsState);
    graph.setItemState(sourceNode, ItemState.ActiveAnchorPoints, true);
  },

  handleNodeMouseLeave(e) {
    const {
      graph,
      edge
    } = this;
    const {
      item
    } = e;

    if (!edge) {
      item.set('anchorPointsState', []);
      graph.setItemState(item, ItemState.ActiveAnchorPoints, false);
    }
  },

  handleNodeMouseDown(e) {
    if (!this.shouldBegin(e) || !this.shouldAddDelegateEdge(e)) {
      return;
    }

    const {
      graph,
      edgeType,
      getAnchorPointStateOfTargetNode
    } = this;
    const {
      target
    } = e;
    const sourceNode = e.item;
    const sourceNodeId = sourceNode.getModel().id;
    const sourceAnchorPointIndex = target.get('anchorPointIndex');
    const sourceAnchorPoint = sourceNode.getAnchorPoints()[sourceAnchorPointIndex];
    const model = {
      id: edgeType + '-' + guid(),
      type: edgeType,
      source: sourceNodeId,
      sourceAnchor: sourceAnchorPointIndex,
      target: {
        x: e.x,
        y: e.y
      }
    };
    this.edge = graph.addItem(ItemType.Edge, model);
    graph.getNodes().forEach(targetNode => {
      if (targetNode.getModel().id === sourceNodeId) {
        return;
      }

      const targetAnchorPoints = targetNode.getAnchorPoints();
      const targetAnchorPointsState = [];
      targetAnchorPoints.forEach(targetAnchorPoint => {
        targetAnchorPointsState.push(getAnchorPointStateOfTargetNode(sourceNode, sourceAnchorPoint, targetNode, targetAnchorPoint));
      });
      targetNode.set('anchorPointsState', targetAnchorPointsState);
      graph.setItemState(targetNode, ItemState.ActiveAnchorPoints, true);
    });
  },

  handleMouseMove(e) {
    const {
      graph,
      edge
    } = this;

    if (!edge) {
      return;
    }

    if (this.canFindTargetAnchorPoint(e)) {
      const {
        item,
        target
      } = e;
      const targetId = item.getModel().id;
      const targetAnchor = target.get('anchorPointIndex');
      graph.updateItem(edge, {
        target: targetId,
        targetAnchor
      });
    } else {
      graph.updateItem(edge, {
        target: {
          x: e.x,
          y: e.y
        },
        targetAnchor: undefined
      });
    }
  },

  handleMouseUp() {
    const {
      graph,
      edge
    } = this;

    if (!edge) {
      return;
    }

    if (!this.shouldAddRealEdge()) {
      graph.removeItem(this.edge);
    }

    this.edge = null;
    graph.getNodes().forEach(node => {
      node.set('anchorPointsState', []);
      graph.setItemState(node, ItemState.ActiveAnchorPoints, false);
    });
  }

};
behaviorManager.register('drag-add-edge', dragAddEdgeBehavior);

const Flow = props => {
  const {
    graph
  } = useEditorContext();
  const graphRef = React.useRef(graph);
  React.useEffect(() => {
    graphRef.current = graph;
  }, [graph]);
  const canDragNode = React.useCallback(e => {
    return !['anchor', 'banAnchor'].some(item => item === e.target.get('className'));
  }, []);
  const canDragOrZoomCanvas = React.useCallback(() => {
    const graph = graphRef.current;

    if (!graph) {
      return false;
    }

    return global.plugin.itemPopover.state === 'hide' && global.plugin.contextMenu.state === 'hide' && global.plugin.editableLabel.state === 'hide';
  }, []);

  const parseData = data => {
    const {
      nodes,
      edges
    } = data;
    [...nodes, ...edges].forEach(item => {
      const {
        id
      } = item;

      if (!id) {
        item.id = item.type + '-' + guid();
      }
    });
  };

  const config = React.useMemo(() => {
    const modes = merge(behaviorManager.getRegisteredBehaviors(), {
      default: {
        'drag-node': {
          type: 'drag-node',
          enableDelegate: true,
          shouldBegin: canDragNode
        },
        'drag-canvas': {
          type: 'drag-canvas',
          shouldBegin: canDragOrZoomCanvas,
          shouldUpdate: canDragOrZoomCanvas
        },
        'zoom-canvas': {
          type: 'zoom-canvas',
          shouldUpdate: canDragOrZoomCanvas
        },
        'recall-edge': 'recall-edge',
        'brush-select': 'brush-select'
      }
    });
    const {
      customModes
    } = props;
    Object.keys(modes).forEach(mode => {
      const behaviors = modes[mode];
      modes[mode] = Object.values(customModes ? customModes(mode, behaviors) : behaviors);
    });
    return merge({
      defaultNode: {
        type: 'task-node'
      },
      defaultEdge: {
        type: 'flowEdge'
      },
      modes
    }, props.graphConfig || {});
  }, []);
  return /*#__PURE__*/React__default.createElement(GraphComponent, Object.assign({
    data: props.data,
    config: config,
    parseData: parseData
  }, omit(props, ['customModes'])));
};

const Item = props => {
  let {
    graph
  } = useEditorContext();

  const handleMouseDown = () => {
    const {
      type = ItemType.Node,
      model
    } = props;

    if (type === ItemType.Node) {
      global.component.itemPanel.model = model;
      graph.setMode(GraphMode.AddNode);
    }
  };

  return /*#__PURE__*/React__default.createElement("div", Object.assign({}, pick(props, ['style', 'className']), {
    onMouseDown: handleMouseDown
  }), props.children);
};

const ItemPanel = props => {
  let {
    graph
  } = useEditorContext();
  const handleMouseUp = React.useCallback(() => {
    if (graph.getCurrentMode() === GraphMode.Default) {
      return;
    }

    const group = graph.get('group');
    const shape = group.findByClassName(global.component.itemPanel.delegateShapeClassName);

    if (shape) {
      shape.remove(true);
      graph.paint();
    }

    global.component.itemPanel.model = null;
    graph.setMode(GraphMode.Default);
  }, [graph]);
  React.useEffect(() => {
    document.addEventListener('mouseup', handleMouseUp, false);
    return () => {
      document.removeEventListener('mouseup', handleMouseUp, false);
    };
  }, [handleMouseUp]);
  return /*#__PURE__*/React__default.createElement("div", Object.assign({}, pick(props, ['styles', 'className'])), props.children);
};

const EditableLabel = props => {
  const {
    labelClassName = 'node-label',
    labelMaxWidth = 100
  } = props;
  let {
    graph,
    executeCommand
  } = useEditorContext();
  const divRef = React.useRef(null);
  const [visible, setVisible] = React.useState(false);
  const showEditableLabel = React.useCallback(() => {
    global.plugin.editableLabel.state = 'show';
    setVisible(true);
    setTimeout(() => {
      if (divRef.current) {
        divRef.current.focus();
        document.execCommand('selectAll', false, null);
      }
    }, 0);
  }, []);
  const hideEditableLabel = React.useCallback(() => {
    global.plugin.editableLabel.state = 'hide';
    setVisible(false);
  }, []);

  const update = () => {
    if (graph && divRef.current) {
      const node = getSelectedNodes(graph)[0];
      const model = node.getModel();
      const {
        textContent: label
      } = divRef.current;

      if (label === model.label) {
        return;
      }

      executeCommand('update', {
        id: model.id,
        updateModel: {
          label
        },
        forceRefreshLayout: false
      });
    }
  };

  const handleBlur = () => {
    update();
    hideEditableLabel();
  };

  const handleKeyDown = e => {
    e.stopPropagation();
    const {
      key
    } = e;

    if (['Tab'].includes(key)) {
      e.preventDefault();
    }

    if (['Enter', 'Escape', 'Tab'].includes(key)) {
      update();
      hideEditableLabel();
    }
  };

  React.useEffect(() => {
    const handler = ({
      labelState
    }) => {
      if (labelState === LabelState.Show) {
        showEditableLabel();
      } else {
        hideEditableLabel();
      }
    };

    if (graph) {
      graph.on(EditorEvent.onLabelStateChange, handler);
      graph.on(GraphNodeEvent.onNodeDoubleClick, showEditableLabel);
    }

    return () => {
      if (graph) {
        graph.off(EditorEvent.onLabelStateChange, handler);
        graph.off(GraphNodeEvent.onNodeDoubleClick, showEditableLabel);
      }
    };
  }, [graph]);

  if (!graph) {
    return null;
  }

  const mode = graph.getCurrentMode();
  const zoom = graph.getZoom();

  if (mode === GraphMode.Readonly) {
    return null;
  }

  const node = getSelectedNodes(graph)[0];

  if (!node) {
    return null;
  }

  const model = node.getModel();
  const group = node.getContainer();
  const label = model.label;
  const labelShape = group.findByClassName(labelClassName);

  if (!labelShape) {
    return null;
  }

  if (!visible) {
    return null;
  } // Get the label offset


  const {
    x: relativeX,
    y: relativeY
  } = labelShape.getBBox();
  const {
    x: absoluteX,
    y: absoluteY
  } = G6.Util.applyMatrix({
    x: relativeX,
    y: relativeY
  }, node.getContainer().getMatrix());
  const {
    x: left,
    y: top
  } = graph.getCanvasByPoint(absoluteX, absoluteY); // Get the label size

  const {
    width,
    height
  } = labelShape.getBBox(); // Get the label font

  const font = labelShape.attr('font');
  const style = {
    position: 'absolute',
    top,
    left,
    width: 'auto',
    height: 'auto',
    minWidth: width,
    minHeight: height,
    maxWidth: labelMaxWidth,
    font,
    background: 'white',
    border: '1px solid #1890ff',
    outline: 'none',
    transform: `scale(${zoom})`,
    transformOrigin: 'left top'
  };
  return /*#__PURE__*/ReactDOM.createPortal( /*#__PURE__*/React__default.createElement("div", {
    ref: divRef,
    style: style,
    contentEditable: true,
    onBlur: handleBlur,
    onKeyDown: handleKeyDown,
    suppressContentEditableWarning: true
  }, label), graph.get('container'));
};

var ItemPopoverType;

(function (ItemPopoverType) {
  ItemPopoverType["Node"] = "node";
  ItemPopoverType["Edge"] = "edge";
})(ItemPopoverType || (ItemPopoverType = {}));

const ItemPopover = props => {
  const {
    type = ItemPopoverType.Node,
    title,
    content,
    mouseEnterDelay = 250,
    mouseLeaveDelay = 250,
    placement = 'top'
  } = props;
  const [item, setItem] = React.useState(null);
  const [visible, setVisible] = React.useState(false);
  const [position, setPosition] = React.useState(null);
  let {
    graph
  } = useEditorContext();
  const mouseEnterTimoutRef = React.useRef(0);
  const mouseLeaveTimoutRef = React.useRef(0);
  const showItemPopover = React.useCallback(item => {
    global.plugin.itemPopover.state = 'show';
    const {
      minX,
      minY,
      maxX,
      maxY,
      centerX,
      centerY
    } = item.getBBox();
    const {
      x: itemMinX,
      y: itemMinY
    } = graph.getCanvasByPoint(minX, minY);
    const {
      x: itemMaxX,
      y: itemMaxY
    } = graph.getCanvasByPoint(maxX, maxY);
    const {
      x: itemCenterX,
      y: itemCenterY
    } = graph.getCanvasByPoint(centerX, centerY);
    const position = {
      minX: itemMinX,
      minY: itemMinY,
      maxX: itemMaxX,
      maxY: itemMaxY,
      centerX: itemCenterX,
      centerY: itemCenterY
    };
    setItem(item);
    setPosition(position);
    setVisible(true);
  }, [graph]);
  const hideItemPopover = React.useCallback(() => {
    global.plugin.itemPopover.state = 'hide';
    setVisible(false);
    setPosition(null);
  }, []);
  React.useEffect(() => {
    const handleMouseEnter = ({
      item
    }) => {
      clearTimeout(mouseEnterTimoutRef.current);
      mouseEnterTimoutRef.current = delay(showItemPopover, mouseEnterDelay, item);
    };

    const handleMouseLeave = ({
      item
    }) => {
      clearTimeout(mouseLeaveTimoutRef.current);
      mouseLeaveTimoutRef.current = delay(hideItemPopover, mouseLeaveDelay, item);
    };

    if (graph) {
      graph.on(GraphNodeEvent.onNodeMouseEnter, handleMouseEnter);
      graph.on(GraphNodeEvent.onNodeMouseLeave, handleMouseLeave);
    }

    return () => {
      if (graph) {
        graph.off(GraphNodeEvent.onNodeMouseEnter, handleMouseEnter);
        graph.off(GraphNodeEvent.onNodeMouseLeave, handleMouseLeave);
      }
    };
  }, [graph, type, mouseEnterDelay, mouseLeaveDelay]);

  if (!visible || !position || !item) {
    return null;
  }

  let styles;

  if (placement === 'top') {
    styles = {
      position: "absolute",
      left: position.centerX,
      top: position.minY
    };
  } else if (placement === 'topLeft') {
    styles = {
      position: "absolute",
      left: position.minX,
      top: position.minY
    };
  } else if (placement === 'topRight') {
    styles = {
      position: "absolute",
      left: position.maxX,
      top: position.minY
    };
  } else if (placement === 'left') {
    styles = {
      position: "absolute",
      left: position.minX,
      top: position.centerY
    };
  } else if (placement === 'right') {
    styles = {
      position: "absolute",
      left: position.maxX,
      top: position.centerY
    };
  } else if (placement === 'bottom') {
    styles = {
      position: "absolute",
      left: position.centerX,
      top: position.maxY
    };
  } else if (placement === 'bottomLeft') {
    styles = {
      position: "absolute",
      left: position.minX,
      top: position.maxY
    };
  } else {
    styles = {
      position: "absolute",
      left: position.maxX,
      top: position.maxY
    };
  }

  return /*#__PURE__*/ReactDOM.createPortal( /*#__PURE__*/React__default.createElement(Popover, {
    visible: true,
    title: title(item),
    content: content(item),
    placement: placement
  }, /*#__PURE__*/React__default.createElement("div", {
    style: styles
  })), graph.get('container'));
};

const DetailContext = React__default.createContext({});
const DetailContextProvider = DetailContext.Provider;
function useFlowDetail() {
  return React__default.useContext(DetailContext);
}

const DetailPanel = props => {
  const [graphState, setGraphState] = React.useState(GraphState.CanvasSelected);
  let {
    graph
  } = useEditorContext();
  const [count, add] = React.useReducer(status => {
    return status + 1;
  }, 1);
  React.useEffect(() => {
    const handler = ({
      graphState
    }) => {
      setGraphState(graphState);
      add();
    };

    graph === null || graph === void 0 ? void 0 : graph.on(EditorEvent.onGraphStateChange, handler);
    return () => {
      graph === null || graph === void 0 ? void 0 : graph.off(EditorEvent.onGraphStateChange, handler);
    };
  }, [graph]);

  if (!graph) {
    return null;
  }

  const isNode = graphState === GraphState.NodeSelected;
  const isEdge = graphState === GraphState.EdgeSelected;
  const isCanvas = graphState === GraphState.CanvasSelected || graphState === GraphState.MultiSelected;
  let item = null;

  if (isNode) {
    let nodes = getSelectedNodes(graph);

    if ((nodes === null || nodes === void 0 ? void 0 : nodes.length) > 0) {
      item = nodes[0];
    }
  } else if (isEdge) {
    let edges = getSelectedEdges(graph);

    if ((edges === null || edges === void 0 ? void 0 : edges.length) > 0) {
      item = edges[0];
    }
  }

  return /*#__PURE__*/React__default.createElement(DetailContextProvider, {
    key: count,
    value: {
      isEdge,
      isNode,
      isCanvas,
      item
    }
  }, props.children);
};

const Command = props => {
  const {
    name,
    className = 'command',
    disabledClassName = 'command-disabled'
  } = props;
  const [disabled, setDisabled] = React.useState(false);
  let {
    graph,
    executeCommand
  } = useEditorContext();
  React.useEffect(() => {
    if (graph) {
      const commandManager = graph.get('commandManager');

      const stateChangeHandler = () => {
        setDisabled(!commandManager.canExecute(graph, name));
      };

      setDisabled(!commandManager.canExecute(graph, name));
      graph.on(EditorEvent.onGraphStateChange, stateChangeHandler);
      return () => {
        graph.off(EditorEvent.onGraphStateChange, stateChangeHandler);
      };
    }
  }, [graph]);

  const handleClick = () => {
    executeCommand(name);
  };

  if (!graph) {
    return null;
  }

  return /*#__PURE__*/React__default.createElement("div", {
    className: classNames(className, {
      [disabledClassName]: disabled
    }),
    onClick: handleClick
  }, props.children);
};

const RegisterNode = props => {
  React.useMemo(() => {
    const {
      name,
      config,
      extend
    } = props;
    G6.registerNode(name, config, extend);
  }, []);
  return null;
};

const RegisterEdge = props => {
  React.useMemo(() => {
    const {
      name,
      config,
      extend
    } = props;
    G6.registerEdge(name, config, extend);
  }, []);
  return null;
};

const RegisterCommand = props => {
  let {
    commandManager
  } = useEditorContext();
  React.useMemo(() => {
    const {
      name,
      command
    } = props;

    if (commandManager) {
      commandManager.register(name, command);
    }
  }, [commandManager]);
  return null;
};

const RegisterBehavior = props => {
  React.useMemo(() => {
    const {
      name,
      behavior
    } = props;
    behaviorManager.register(name, behavior);
  }, []);
  return null;
};

exports.G6 = G6;
exports.Command = Command;
exports.CommandManager = CommandManager;
exports.DetailPanel = DetailPanel;
exports.EditableLabel = EditableLabel;
exports.Flow = Flow;
exports.Item = Item;
exports.ItemPanel = ItemPanel;
exports.ItemPopover = ItemPopover;
exports.RegisterBehavior = RegisterBehavior;
exports.RegisterCommand = RegisterCommand;
exports.RegisterEdge = RegisterEdge;
exports.RegisterNode = RegisterNode;
exports.baseCommand = baseCommand;
exports.behaviorManager = behaviorManager;
exports.constants = index;
exports.default = Editor;
exports.global = global;
exports.setAnchorPointsState = setAnchorPointsState;
exports.useEditorContext = useEditorContext;
exports.useFlowDetail = useFlowDetail;
